@@ -1,7 +1,4 @@
 """ Tidy3d package imports"""
-from concurrent.futures import ProcessPoolExecutor, process
-
-from rich import pretty, traceback
 
 # grid
 from .components.grid.grid import Grid, Coords
@@ -9,10 +6,12 @@ from .components.grid.grid_spec import GridSpec, UniformGrid, CustomGrid, AutoGr
 
 # geometry
 from .components.geometry import Box, Sphere, Cylinder, PolySlab, GeometryGroup
+from .components.geometry import TriangleMesh
 
 # medium
-from .components.medium import Medium, PoleResidue, AnisotropicMedium, PEC, PECMedium
+from .components.medium import Medium, PoleResidue, AnisotropicMedium, PEC, PECMedium, Medium2D
 from .components.medium import Sellmeier, Debye, Drude, Lorentz
+from .components.medium import CustomMedium
 
 # structures
 from .components.structure import Structure, MeshOverrideStructure
@@ -27,7 +26,7 @@ from .components.apodization import ApodizationSpec
 from .components.source import GaussianPulse, ContinuousWave
 from .components.source import UniformCurrentSource, PlaneWave, ModeSource, PointDipole
 from .components.source import GaussianBeam, AstigmaticGaussianBeam
-from .components.source import CustomFieldSource
+from .components.source import CustomFieldSource, TFSF
 
 # monitors
 from .components.monitor import FieldMonitor, FieldTimeMonitor, FluxMonitor, FluxTimeMonitor
@@ -72,11 +71,12 @@ from .components.boundary import DefaultPMLParameters, DefaultStablePMLParameter
 from .components.boundary import DefaultAbsorberParameters
 
 # constants imported as `C_0 = td.C_0` or `td.constants.C_0`
-from .constants import C_0, ETA_0, HBAR, EPSILON_0, MU_0, Q_e, inf
+from .constants import C_0, ETA_0, HBAR, EPSILON_0, MU_0, Q_e, K_B, inf
 
 # material library dict imported as `from tidy3d import material_library`
 # get material `mat` and variant `var` as `material_library[mat][var]`
 from .material_library.material_library import material_library
+from .material_library.parametric_materials import Graphene
 
 # for docs
 from .components.medium import AbstractMedium
@@ -85,8 +85,7 @@ from .components.source import Source, SourceTime
 from .components.monitor import Monitor
 from .components.grid.grid import YeeGrid, FieldGrid, Coords1D
 
-# logging
-from .log import log, set_logging_file, WebError
+from .log import log, set_logging_file, set_logging_console
 
 # config
 from .config import config
@@ -97,6 +96,7 @@ from .version import __version__, PIP_NAME
 # updater
 from .updater import Updater
 
+
 # warn if this was uploaded to the beta PyPI, also sets the pyPI repository that we upload to
 if "beta" in PIP_NAME:
     log.warning(
@@ -114,7 +114,4 @@ def set_logging_level(level: str) -> None:
     )
 
 
-# make all stdout and errors pretty
-# pretty.install()
-# traceback.install()
 log.info(f"Using client version: {__version__}")
