@@ -8,12 +8,13 @@ import pydantic as pd
 import numpy as np
 
 from .base import Tidy3dBaseModel, cached_property
-from .types import Complex, Axis, Literal
-from .source import GaussianBeam, ModeSource, PlaneWave
+from .types import Complex, Axis, TYPE_TAG_STR
+from .source import GaussianBeam, ModeSource, PlaneWave, TFSF
 from .medium import Medium
 
-from ..log import log, SetupError, DataError, ValidationError
 from ..constants import EPSILON_0, MU_0, PML_SIGMA
+from ..exceptions import SetupError, DataError
+from ..log import log
 
 
 class BoundaryEdge(ABC, Tidy3dBaseModel):
@@ -40,7 +41,7 @@ class PMCBoundary(BoundaryEdge):
 # """ Bloch boundary """
 
 # sources from which Bloch boundary conditions can be defined
-BlochSourceType = Union[GaussianBeam, ModeSource, PlaneWave]
+BlochSourceType = Union[GaussianBeam, ModeSource, PlaneWave, TFSF]
 
 
 class BlochBoundary(BoundaryEdge):
@@ -51,7 +52,7 @@ class BlochBoundary(BoundaryEdge):
     >>> bloch = BlochBoundary(bloch_vec=1)
     """
 
-    bloch_vec: Complex = pd.Field(
+    bloch_vec: float = pd.Field(
         ...,
         title="Normalized Bloch vector component",
         description="Normalized component of the Bloch vector in units of "
@@ -102,8 +103,8 @@ class BlochBoundary(BoundaryEdge):
 
         if not isinstance(source, BlochSourceType.__args__):
             raise SetupError(
-                "The `source` parameter must be `GaussianBeam`, `ModeSource`, or `PlaneWave` "
-                "in order to define a Bloch boundary condition."
+                "The `source` parameter must be `GaussianBeam`, `ModeSource`, `PlaneWave`, "
+                "or `TFSF` in order to define a Bloch boundary condition."
             )
 
         if axis == source.injection_axis:
@@ -143,13 +144,6 @@ class BlochBoundary(BoundaryEdge):
         bloch_vec = domain_size * k_global[axis]
         return cls(bloch_vec=bloch_vec)
 
-    @pd.validator("bloch_vec", always=True)
-    def _zero_imaginary_part(cls, val):
-        """Make sure the imaginary part of the bloch vector is 0."""
-        if np.imag(val) != 0:
-            raise ValidationError("Bloch vector must be real.")
-        return val
-
 
 # """ Absorber parameters """
 
@@ -335,7 +329,6 @@ PMLTypes = Union[PML, StablePML, Absorber, None]
 
 # types of boundaries that can be used in Simulation
 
-Symmetry = Literal[0, -1, 1]
 BoundaryEdgeType = Union[
     Periodic, PECBoundary, PMCBoundary, PML, StablePML, Absorber, BlochBoundary
 ]
@@ -353,14 +346,33 @@ class Boundary(Tidy3dBaseModel):
         Periodic(),
         title="Plus BC",
         description="Boundary condition on the plus side along a dimension.",
+        discriminator=TYPE_TAG_STR,
     )
 
     minus: BoundaryEdgeType = pd.Field(
         Periodic(),
         title="Minus BC",
         description="Boundary condition on the minus side along a dimension.",
+        discriminator=TYPE_TAG_STR,
     )
 
+    # TODO: remove for 2.0
+    @pd.root_validator(pre=True)
+    def _deprecation_2_0_missing_defaults(cls, values):
+        """Raise deprecation warning if a default `plus` or `minus` is used."""
+
+        fields_use_default = [field for field in ("plus", "minus") if values.get(field) is None]
+
+        for field in fields_use_default:
+            log.warning(
+                f"'Boundary.{field}' uses default value, which is 'Periodic()' "
+                "but will change to 'PML()' in Tidy3D version 2.0. "
+                "We recommend explicitly setting all boundary conditions "
+                "ahead of this release to avoid unexpected results."
+            )
+
+        return values
+
     @pd.root_validator(skip_on_failure=True)
     def bloch_on_both_sides(cls, values):
         """Error if a Bloch boundary is applied on only one side."""
@@ -403,7 +415,7 @@ class Boundary(Tidy3dBaseModel):
         if switched:
             values.update({"plus": plus, "minus": minus})
             log.warning(
-                "A periodic boundary condition was specified on the opposide side of a perfect "
+                "A periodic boundary condition was specified on the opposite side of a perfect "
                 "electric or magnetic conductor boundary. This periodic boundary condition will "
                 "be replaced by the perfect electric or magnetic conductor across from it."
             )
@@ -563,21 +575,46 @@ class BoundarySpec(Tidy3dBaseModel):
     """Specifies boundary conditions on each side of the domain and along each dimension."""
 
     x: Boundary = pd.Field(
-        Boundary(),
+        None,
         title="Boundary condition along x.",
-        description="Boundary condition on the plus and minus sides along the x axis.",
+        description="Boundary condition on the plus and minus sides along the x axis. "
+        "If `None`, periodic boundaries are applied. Default will change to PML in 2.0 "
+        "so explicitly setting the boundaries is recommended.",
     )
+
     y: Boundary = pd.Field(
-        Boundary(),
+        None,
         title="Boundary condition along y.",
-        description="Boundary condition on the plus and minus sides along the y axis.",
+        description="Boundary condition on the plus and minus sides along the y axis. "
+        "If `None`, periodic boundaries are applied. Default will change to PML in 2.0 "
+        "so explicitly setting the boundaries is recommended.",
     )
+
     z: Boundary = pd.Field(
-        Boundary(),
+        None,
         title="Boundary condition along z.",
-        description="Boundary condition on the plus and minus sides along the z axis.",
+        description="Boundary condition on the plus and minus sides along the z axis. "
+        "If `None`, periodic boundaries are applied. Default will change to PML in 2.0 "
+        "so explicitly setting the boundaries is recommended.",
     )
 
+    # TODO: remove for 2.0
+    @pd.root_validator(pre=True)
+    def _deprecation_2_0_missing_defaults(cls, values):
+        """Raise deprecation warning if a default boundary condition is used."""
+
+        for field in "xyz":
+            if values.get(field) is None:
+                log.warning(
+                    f"'BoundarySpec.{field}' uses default value, which is 'Periodic()' "
+                    "but will change to 'PML()' in Tidy3D version 2.0. "
+                    "We recommend explicitly setting all boundary conditions "
+                    "ahead of this release to avoid unexpected results."
+                )
+                values[field] = Boundary.periodic()
+
+        return values
+
     def __getitem__(self, field_name: str) -> Boundary:
         """Get the :class:`Boundary` field by name (``boundary_spec[field_name]``).
 
@@ -599,30 +636,9 @@ class BoundarySpec(Tidy3dBaseModel):
             return self.z
         raise DataError(f"field_name '{field_name}' not found")
 
-    def __setitem__(self, field_name: str, field: Boundary):
-        """Set the :class:`Boundary` field by name (``boundary_spec[field_name]=field``).
-
-        Parameters
-        ----------
-        field_name : ``str``
-            Name of the axis, eg. "y" along which :class:`Boundary` is to be set.
-        field : :class:`Boundary`
-            Boundary condition to set along the named axis.
-        """
-        if field_name == "x":
-            self.x = field
-            return
-        if field_name == "y":
-            self.y = field
-            return
-        if field_name == "z":
-            self.z = field
-            return
-        raise DataError(f"field_name '{field_name}' not found")
-
     @classmethod
     def pml(cls, x: bool = False, y: bool = False, z: bool = False):
-        """Default PML along specified directions
+        """PML along specified directions
 
         Parameters
         ----------
@@ -638,9 +654,9 @@ class BoundarySpec(Tidy3dBaseModel):
         >>> boundaries = BoundarySpec.pml(y=True)
         """
         return cls(
-            x=Boundary.pml() if x else Boundary.periodic(),
-            y=Boundary.pml() if y else Boundary.periodic(),
-            z=Boundary.pml() if z else Boundary.periodic(),
+            x=Boundary.pml() if x else None,
+            y=Boundary.pml() if y else None,
+            z=Boundary.pml() if z else None,
         )
 
     @classmethod
@@ -661,9 +677,9 @@ class BoundarySpec(Tidy3dBaseModel):
         >>> boundaries = BoundarySpec.pec(x=True, z=True)
         """
         return cls(
-            x=Boundary.pec() if x else Boundary.periodic(),
-            y=Boundary.pec() if y else Boundary.periodic(),
-            z=Boundary.pec() if z else Boundary.periodic(),
+            x=Boundary.pec() if x else None,
+            y=Boundary.pec() if y else None,
+            z=Boundary.pec() if z else None,
         )
 
     @classmethod
@@ -684,9 +700,9 @@ class BoundarySpec(Tidy3dBaseModel):
         >>> boundaries = BoundarySpec.pmc(x=True, z=True)
         """
         return cls(
-            x=Boundary.pmc() if x else Boundary.periodic(),
-            y=Boundary.pmc() if y else Boundary.periodic(),
-            z=Boundary.pmc() if z else Boundary.periodic(),
+            x=Boundary.pmc() if x else None,
+            y=Boundary.pmc() if y else None,
+            z=Boundary.pmc() if z else None,
         )
 
     @classmethod
@@ -716,3 +732,16 @@ class BoundarySpec(Tidy3dBaseModel):
             (self.y.minus, self.y.plus),
             (self.z.minus, self.z.plus),
         ]
+
+    @cached_property
+    def flipped_bloch_vecs(self) -> BoundarySpec:
+        """Return a copy of the instance where all Bloch vectors are multiplied by -1."""
+        bound_dims = dict(x=self.x.copy(), y=self.y.copy(), z=self.z.copy())
+        for dim_key, bound_dim in bound_dims.items():
+            bound_edges = dict(plus=bound_dim.plus.copy(), minus=bound_dim.minus.copy())
+            for edge_key, bound_edge in bound_edges.items():
+                if isinstance(bound_edge, BlochBoundary):
+                    new_bloch_vec = -1 * bound_edge.bloch_vec
+                    bound_edges[edge_key] = bound_edge.copy(update=dict(bloch_vec=new_bloch_vec))
+            bound_dims[dim_key] = bound_edges
+        return self.copy(update=bound_dims)
