@@ -5,7 +5,7 @@ from typing import Union, Tuple
 import pydantic
 import numpy as np
 
-from .types import Ax, EMField, ArrayLike, FreqArray
+from .types import Ax, EMField, ArrayFloat1D, FreqArray
 from .types import Literal, Direction, Coordinate, Axis, ObsGridArray
 from .geometry import Box
 from .validators import assert_plane
@@ -49,7 +49,7 @@ class Monitor(Box, ABC):
         return Box(center=self.center, size=self.size)
 
     @abstractmethod
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization.
 
         Parameters
@@ -126,7 +126,7 @@ class TimeMonitor(Monitor, ABC):
             raise SetupError("Monitor start time is greater than stop time.")
         return val
 
-    def time_inds(self, tmesh: ArrayLike[float, 1]) -> Tuple[int, int]:
+    def time_inds(self, tmesh: ArrayFloat1D) -> Tuple[int, int]:
         """Compute the starting and stopping index of the monitor in a given discrete time mesh."""
 
         tmesh = np.array(tmesh)
@@ -155,7 +155,7 @@ class TimeMonitor(Monitor, ABC):
             tind_beg = tbeg[0] if tbeg.size > 0 else tind_end
         return (tind_beg, tind_end)
 
-    def num_steps(self, tmesh: ArrayLike[float, 1]) -> int:
+    def num_steps(self, tmesh: ArrayFloat1D) -> int:
         """Compute number of time steps for a time monitor."""
 
         tind_beg, tind_end = self.time_inds(tmesh)
@@ -290,7 +290,7 @@ class FieldMonitor(AbstractFieldMonitor, FreqMonitor):
     ...     name='steady_state_monitor')
     """
 
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization."""
         # stores 1 complex number per grid cell, per frequency, per field
         return BYTES_COMPLEX * num_cells * len(self.freqs) * len(self.fields)
@@ -311,7 +311,7 @@ class FieldTimeMonitor(AbstractFieldMonitor, TimeMonitor):
     ...     name='movie_monitor')
     """
 
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization."""
         # stores 1 real number per grid cell, per time step, per field
         num_steps = self.num_steps(tmesh)
@@ -324,6 +324,11 @@ class PermittivityMonitor(FreqMonitor):
     :class:`.FieldMonitor` of the same geometry: the permittivity values are saved at the
     Yee grid locations, and can be interpolated to any point inside the monitor.
 
+    Note
+    ----
+    If 2D materials are present, then the permittivity values correspond to the
+    volumetric equivalent of the 2D materials.
+
     Example
     -------
     >>> monitor = PermittivityMonitor(
@@ -333,7 +338,7 @@ class PermittivityMonitor(FreqMonitor):
     ...     name='eps_monitor')
     """
 
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization."""
         # stores 3 complex number per grid cell, per frequency
         return BYTES_COMPLEX * num_cells * len(self.freqs) * 3
@@ -361,12 +366,7 @@ class SurfaceIntegrationMonitor(Monitor, ABC):
     def integration_surfaces(self):
         """Surfaces of the monitor where fields will be recorded for subsequent integration."""
         if self.size.count(0.0) == 0:
-            monitor_dict = self.dict()
-            exclude_surfaces = monitor_dict.pop("exclude_surfaces", None)
-            surface_monitors = self.surfaces(**monitor_dict)
-            if exclude_surfaces:
-                return [mnt for mnt in surface_monitors if mnt.name[-2:] not in exclude_surfaces]
-            return surface_monitors
+            return self.surfaces_with_exclusion(**self.dict())
         return [self]
 
     @pydantic.root_validator(skip_on_failure=True)
@@ -423,7 +423,7 @@ class FluxMonitor(AbstractFluxMonitor, FreqMonitor):
     ...     name='flux_monitor')
     """
 
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization."""
         # stores 1 real number per frequency
         return BYTES_REAL * len(self.freqs)
@@ -447,7 +447,7 @@ class FluxTimeMonitor(AbstractFluxMonitor, TimeMonitor):
     ...     name='flux_vs_time')
     """
 
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization."""
         # stores 1 real number per time step
         num_steps = self.num_steps(tmesh)
@@ -617,7 +617,7 @@ class FieldProjectionAngleMonitor(AbstractFieldProjectionMonitor):
         units=RADIAN,
     )
 
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization."""
         # stores 1 complex number per pair of angles, per frequency,
         # for Er, Etheta, Ephi, Hr, Htheta, and Hphi (6 components)
@@ -677,7 +677,7 @@ class FieldProjectionCartesianMonitor(AbstractFieldProjectionMonitor):
         units=MICROMETER,
     )
 
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization."""
         # stores 1 complex number per pair of grid points, per frequency,
         # for Er, Etheta, Ephi, Hr, Htheta, and Hphi (6 components)
@@ -745,7 +745,7 @@ class FieldProjectionKSpaceMonitor(AbstractFieldProjectionMonitor):
             raise SetupError(f"Entries of 'uy' must lie in the range [-1, 1] for monitor {name}.")
         return values
 
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization."""
         # stores 1 complex number per pair of grid points, per frequency,
         # for Er, Etheta, Ephi, Hr, Htheta, and Hphi (6 components)
@@ -785,7 +785,7 @@ class DiffractionMonitor(PlanarMonitor, FreqMonitor):
             )
         return val
 
-    def storage_size(self, num_cells: int, tmesh: ArrayLike[float, 1]) -> int:
+    def storage_size(self, num_cells: int, tmesh: ArrayFloat1D) -> int:
         """Size of monitor storage given the number of points after discretization."""
         # assumes 1 diffraction order per frequency; actual size will be larger
         return BYTES_COMPLEX * len(self.freqs)
