@@ -1,9 +1,11 @@
 # pylint:disable=unused-argument
 """ Defines various validation functions that get used to ensure inputs are legit """
+from typing import Any
+
 import pydantic
 
 from .geometry import Box
-from ..log import ValidationError, SetupError
+from ..exceptions import ValidationError, SetupError
 
 """ Explanation of pydantic validators:
 
@@ -38,6 +40,14 @@ from ..log import ValidationError, SetupError
 """
 
 
+def get_value(key: str, values: dict) -> Any:
+    """Grab value from values dictionary. If not present, raise an error before continuing."""
+    val = values.get(key)
+    if val is None:
+        raise ValidationError(f"value {key} not defined, must be present to validate.")
+    return val
+
+
 def assert_plane():
     """makes sure a field's `size` attribute has exactly 1 zero"""
 
@@ -51,6 +61,23 @@ def assert_plane():
     return is_plane
 
 
+def assert_volumetric():
+    """makes sure a field's `size` attribute has no zero entry"""
+
+    @pydantic.validator("size", allow_reuse=True, always=True)
+    def is_volumetric(cls, val):
+        """Raise validation error if volume is 0."""
+        if val.count(0.0) > 0:
+            raise ValidationError(
+                f"'{cls.__name__}' object must be volumetric, given size={val}. "
+                "If intending to make a 2D simulation, please set the size of "
+                f"'{cls.__name__}' along the zero dimension to a dummy non-zero value."
+            )
+        return val
+
+    return is_volumetric
+
+
 def validate_name_str():
     """make sure the name doesnt include [, ] (used for default names)"""
 
@@ -108,18 +135,13 @@ def validate_mode_objects_symmetry(field_name: str):
     return check_symmetry
 
 
-def assert_unique_names(field_name: str, check_mediums=False):
+def assert_unique_names(field_name: str):
     """makes sure all elements of a field have unique .name values"""
 
     @pydantic.validator(field_name, allow_reuse=True, always=True)
     def field_has_unique_names(cls, val, values):
-        """check for intersection of each structure with simulation bounds."""
-        if check_mediums:
-            background = values.get("medium")
-            field_names = [background] + [field.medium.name for field in val if field.medium.name]
-            field_names = [name for name in field_names if ("]" not in name) or ("[" not in name)]
-        else:
-            field_names = [field.name for field in val if field.name]
+        """make sure each element of val has a unique name (if specified)."""
+        field_names = [field.name for field in val if field.name]
         unique_names = set(field_names)
         if len(unique_names) != len(field_names):
             raise SetupError(f"'{field_name}' names are not unique, given {field_names}.")
@@ -151,20 +173,6 @@ def assert_objects_in_sim_bounds(field_name: str):
     return objects_in_sim_bounds
 
 
-def set_names(field_name: str):
-    """set names"""
-
-    @pydantic.validator(field_name, allow_reuse=True, always=True)
-    def set_unique_names(cls, val):
-        """check for intersection of each structure with simulation bounds."""
-        for position_index, geometric_object in enumerate(val):
-            if not geometric_object.name:
-                geometric_object.name = f"{field_name}[{position_index}]"
-        return val
-
-    return set_unique_names
-
-
 def enforce_monitor_fields_present():
     """Make sure all of the fields in the monitor are present in the correponding data."""
 
@@ -172,7 +180,22 @@ def enforce_monitor_fields_present():
     def _contains_fields(cls, values):
         """Make sure the initially specified fields are here."""
         for field_name in values.get("monitor").fields:
-            assert values.get(field_name) is not None, f"missing field {field_name}"
+            if values.get(field_name) is None:
+                raise SetupError(f"missing field {field_name}")
         return values
 
     return _contains_fields
+
+
+def required_if_symmetry_present(field_name: str):
+    """Make a field required (not None) if any non-zero symmetry eigenvalue is present."""
+
+    @pydantic.validator(field_name, allow_reuse=True, always=True)
+    def _make_required(cls, val, values):
+        """Ensure val is not None if the symmetry is non-zero along any dimension."""
+        symmetry = values.get("symmetry")
+        if any(sym_val != 0 for sym_val in symmetry) and val is None:
+            raise SetupError(f"'{field_name}' must be provided if symmetry present.")
+        return val
+
+    return _make_required
